(function() {
  var AbstractMessageExecutor, MaxRetriesExceededException, Promise, _, debug,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require("lodash");

  Promise = require("bluebird");

  MaxRetriesExceededException = require("./maxRetriesExceededException");

  debug = require("debug")("long-task-queue-reader:message-executor");

  module.exports = AbstractMessageExecutor = (function() {
    function AbstractMessageExecutor(arg) {
      var ref;
      ref = arg != null ? arg : {}, this.runner = ref.runner, this.message = ref.message, this.maxRetries = ref.maxRetries, this.fromPoison = ref.fromPoison;
      this.hasReachedMaxRetries = bind(this.hasReachedMaxRetries, this);
    }

    AbstractMessageExecutor.prototype.execute = function() {
      var body;
      body = _.assign({}, this._body_(), {
        messageId: this._messageId_()
      });
      debug("Processing %o", body);
      if (this.hasReachedMaxRetries() && !this.fromPoison) {
        return Promise.reject(new MaxRetriesExceededException(this.message, this.maxRetries));
      }
      return Promise.method(this.runner)(body);
    };

    AbstractMessageExecutor.prototype.hasReachedMaxRetries = function() {
      return this._receiveCount_() > this.maxRetries;
    };

    AbstractMessageExecutor.prototype._body_ = function() {
      throw new Error("not_implemented");
    };

    AbstractMessageExecutor.prototype._receiveCount_ = function() {
      throw new Error("not_implemented");
    };

    AbstractMessageExecutor.prototype._messageId_ = function() {
      throw new Error("not_implemented");
    };

    return AbstractMessageExecutor;

  })();

}).call(this);
