(function() {
  var AzureQueueNode, Promise, Queue, _, debug, retry,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require("lodash");

  AzureQueueNode = require("azure-queue-node");

  Promise = require("bluebird");

  retry = require("bluebird-retry");

  debug = require("debug")("long-task-queue-reader:queue");

  module.exports = Queue = (function() {
    function Queue(arg) {
      var accountKey, accountName, name, queueName;
      accountKey = arg.accountKey, accountName = arg.accountName, queueName = arg.queueName, name = arg.name;
      this.create = bind(this.create, this);
      this.initialize = bind(this.initialize, this);
      this.queueName = queueName || name;
      this.client = this._buildClient(accountName, accountKey);
    }

    Queue.prototype.initialize = function() {
      return Promise.map([this.queueName, this._poisonQueueName()], this.create);
    };

    Queue.prototype.create = function(queue) {
      return this.client.createQueueAsync(queue);
    };

    Queue.prototype.sendToPoison = function(message) {
      return this.pushPoison(message.messageText);
    };

    Queue.prototype.messages = function(opts) {
      if (opts == null) {
        opts = {};
      }
      return this.client.getMessagesAsync(this.queueName, opts).tap(function(messages) {
        return debug("Received Messages: %o", _.map(messages, 'messageText'));
      });
    };

    Queue.prototype.update = function(timeout, arg) {
      var messageId, messageText, popReceipt;
      messageId = arg.messageId, popReceipt = arg.popReceipt, messageText = arg.messageText;
      debug("Updating [timeout: " + timeout + ", messageId: " + messageId + ", popReceipt: " + popReceipt + ", messageText: " + (JSON.stringify(messageText)) + "]");
      return this.client.updateMessageAsync(this.queueName, messageId, popReceipt, timeout, messageText).tap(function(message) {
        return debug("Updated message: %o", message);
      });
    };

    Queue.prototype.remove = function(arg) {
      var messageId, popReceipt;
      messageId = arg.messageId, popReceipt = arg.popReceipt;
      debug("Removing message: [messageId: " + messageId + ", popReceipt: " + popReceipt + "]");
      return retry((function(_this) {
        return function() {
          return _this.client.deleteMessageAsync(_this.queueName, messageId, popReceipt);
        };
      })(this)).tap(function() {
        return debug("Removed messageId: " + messageId);
      });
    };

    Queue.prototype.push = function(message) {
      return this.client.putMessageAsync(this.queueName, message);
    };

    Queue.prototype.pushPoison = function(message) {
      return this.client.putMessageAsync(this._poisonQueueName(), message);
    };

    Queue.prototype._poisonQueueName = function() {
      return this.queueName + "-poison";
    };

    Queue.prototype._buildClient = function(accountName, accountKey) {
      var client;
      client = AzureQueueNode.createClient({
        accountUrl: "https://" + accountName + ".queue.core.windows.net/",
        accountName: accountName,
        accountKey: accountKey,
        base64: true
      });
      return Promise.promisifyAll(client);
    };

    return Queue;

  })();

}).call(this);
