(function() {
  var EventEmitter, KeepAliveMessage, LongTaskQueueReader, MaxRetriesExceededException, Promise, _, convert, eventsToLog, winston,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  _ = require("lodash");

  winston = require("winston");

  Promise = require("bluebird");

  EventEmitter = require("events").EventEmitter;

  convert = require("convert-units");

  KeepAliveMessage = require("./keepAliveMessage");

  MaxRetriesExceededException = require("./maxRetriesExceededException");

  eventsToLog = function(logger) {
    return {
      "job-get-messages": function() {
        return logger.info("Obteniendo mensajes nuevas");
      },
      "job-finish-messages": function() {
        return logger.info("Finalizo la ejecucion de mensajes");
      },
      "message-start": function(message) {
        return logger.info("Iniciando el proceso de un mensaje", message);
      },
      "message-finish": function(message) {
        return logger.info("Finalizo la ejecucion de un proceso", message);
      },
      "message-touch": function(arg) {
        var Body, MessageId, messageId, messageText;
        messageId = arg.messageId, MessageId = arg.MessageId, messageText = arg.messageText, Body = arg.Body;
        return logger.info("Touching " + (messageId || MessageId), messageText || Body);
      },
      "job_error": function(arg) {
        var err, message, method;
        method = arg.method, message = arg.message, err = arg.err;
        return logger.error("An error has ocurred in " + method + "(" + ((message != null ? message.messageId : void 0) || (message != null ? message.MessageId : void 0)) + ")", err);
      }
    };
  };

  module.exports = LongTaskQueueReader = (function(superClass) {
    extend(LongTaskQueueReader, superClass);

    function LongTaskQueueReader(queue, arg, arg1, MessageExecutor, runner, fromPoison) {
      var action, eventName, level, logger, ref, ref1, ref2, ref3, ref4, ref5, transports;
      this.queue = queue;
      this.waitingTime = (ref = arg.waitingTime) != null ? ref : 60, this.visibilityTimeout = (ref1 = arg.visibilityTimeout) != null ? ref1 : 60, this.maxRetries = (ref2 = arg.maxRetries) != null ? ref2 : 10;
      level = (ref3 = arg1.level) != null ? ref3 : "info", transports = (ref4 = arg1.transports) != null ? ref4 : [];
      this.MessageExecutor = MessageExecutor;
      this.runner = runner;
      this.fromPoison = fromPoison;
      this._touch = bind(this._touch, this);
      this._removeSafety = bind(this._removeSafety, this);
      this._createKeepAlive = bind(this._createKeepAlive, this);
      this._sendToPoison = bind(this._sendToPoison, this);
      this._execute = bind(this._execute, this);
      this._buildExecutor = bind(this._buildExecutor, this);
      this._nextTimeout = bind(this._nextTimeout, this);
      this._executePendingSynchronization = bind(this._executePendingSynchronization, this);
      this._initNewTask = bind(this._initNewTask, this);
      this.start = bind(this.start, this);
      logger = new winston.Logger({
        level: level,
        transports: transports
      });
      ref5 = eventsToLog(logger);
      for (eventName in ref5) {
        action = ref5[eventName];
        this.on("" + eventName, action);
      }
    }

    LongTaskQueueReader.prototype.start = function() {
      return this._initNewTask();
    };

    LongTaskQueueReader.prototype._initNewTask = function() {
      return this._executePendingSynchronization().then((function(_this) {
        return function(message) {
          setTimeout(_this._initNewTask, _this._nextTimeout(message));
        };
      })(this));
    };

    LongTaskQueueReader.prototype._executePendingSynchronization = function() {
      this.emit("job-get-messages");
      return this.queue.messages({
        maxMessages: 1,
        visibilityTimeout: this.visibilityTimeout
      }).get(0).tap((function(_this) {
        return function(message) {
          if (message != null) {
            return _this._execute(message);
          }
        };
      })(this)).tap((function(_this) {
        return function() {
          return _this.emit("job-finish-messages");
        };
      })(this))["catch"]((function(_this) {
        return function(err) {
          return _this.emit("job_error", {
            method: "_executePendingSynchronization",
            err: err
          });
        };
      })(this));
    };

    LongTaskQueueReader.prototype._nextTimeout = function(message) {
      if (_.isEmpty(message)) {
        return convert(this.waitingTime).from("s").to("ms");
      } else {
        return 0;
      }
    };

    LongTaskQueueReader.prototype._buildExecutor = function(message) {
      return new this.MessageExecutor({
        runner: this.runner,
        message: message,
        maxRetries: this.maxRetries,
        fromPoison: this.fromPoison
      });
    };

    LongTaskQueueReader.prototype._execute = function(message) {
      var keepAliveMessage;
      keepAliveMessage = this._createKeepAlive(message);
      this.emit("message-start", message);
      keepAliveMessage.start();
      return this._buildExecutor(message).execute().tap((function(_this) {
        return function() {
          return _this._removeSafety(message);
        };
      })(this))["catch"](MaxRetriesExceededException, (function(_this) {
        return function(e) {
          return _this._sendToPoison(message);
        };
      })(this))["catch"]((function(_this) {
        return function(err) {
          return _this.emit("job_error", {
            method: "_execute",
            err: err,
            message: message
          });
        };
      })(this)).tap(function() {
        return keepAliveMessage.destroy();
      }).then((function(_this) {
        return function() {
          return _this.emit("message-finish", message);
        };
      })(this));
    };

    LongTaskQueueReader.prototype._sendToPoison = function(message) {
      return this.queue.sendToPoison(message).then((function(_this) {
        return function() {
          return _this._removeSafety(message);
        };
      })(this));
    };

    LongTaskQueueReader.prototype._createKeepAlive = function(message) {
      return new KeepAliveMessage(message, this.visibilityTimeout, this._touch);
    };

    LongTaskQueueReader.prototype._removeSafety = function(message) {
      return this.queue.remove(message)["catch"]((function(_this) {
        return function(err) {
          return _this.emit("job_error", {
            method: "_removeSafety",
            message: message,
            err: err
          });
        };
      })(this));
    };

    LongTaskQueueReader.prototype._touch = function(message) {
      this.emit("message-touch", message);
      return this.queue.update(this.visibilityTimeout, message).tap(function(response) {
        return _.assign(message, {
          popReceipt: response.popReceipt
        });
      })["catch"]((function(_this) {
        return function(err) {
          return _this.emit("job_error", {
            method: "_touch",
            message: message,
            err: err
          });
        };
      })(this));
    };

    return LongTaskQueueReader;

  })(EventEmitter);

}).call(this);
