(function() {
  var LongTaskQueueReaderBuilder, Promise, QueueProcessor, winston;

  winston = require("winston");

  Promise = require("bluebird");

  QueueProcessor = require("./longTaskQueueReader");

  module.exports = LongTaskQueueReaderBuilder = (function() {
    function LongTaskQueueReaderBuilder(implementation, poison) {
      this.implementation = implementation != null ? implementation : "azure";
      this.poison = poison != null ? poison : false;
      this.transports = [
        new winston.transports.Console({
          timestamp: true
        })
      ];
      this.dependencies = [];
    }

    LongTaskQueueReaderBuilder.prototype.withLogger = function(opts) {
      var Logger, logger;
      Logger = this._internalRequire("logger");
      logger = new Logger(opts);
      this.transports.push(logger.transport());
      this.dependencies.push(logger);
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.withQueue = function(opts) {
      var Queue;
      this.waitingTime = opts.waitingTime, this.timeToUpdateMessage = opts.timeToUpdateMessage;
      Queue = this._internalRequire("queue" + (this.poison ? ".poison" : ""));
      this.queue = new Queue(opts);
      this.dependencies.push(this.queue);
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.fromPoison = function(poison) {
      this.poison = poison;
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.withRunner = function(runner) {
      this.runner = runner;
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.withMaxRetries = function(maxRetries) {
      this.maxRetries = maxRetries;
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.withImplementation = function(implementation) {
      this.implementation = implementation;
      return this;
    };

    LongTaskQueueReaderBuilder.prototype.build = function() {
      return Promise.map(this.dependencies, function(dependency) {
        return dependency.initialize();
      }).then((function(_this) {
        return function() {
          return new QueueProcessor(_this.queue, {
            waitingTime: _this.waitingTime,
            timeToUpdateMessage: _this.timeToUpdateMessage,
            maxRetries: _this.maxRetries
          }, {
            transports: _this.transports
          }, _this._internalRequire("messageExecutor"), _this.runner, _this.poison);
        };
      })(this));
    };

    LongTaskQueueReaderBuilder.prototype._internalRequire = function(name) {
      return require("./" + this.implementation + "/" + name);
    };

    return LongTaskQueueReaderBuilder;

  })();

}).call(this);
